"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2022-Present Chen An
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypedTrigger = void 0;
class TypedTrigger {
    constructor() {
        Object.defineProperty(this, "_handlerCollection", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
    }
    on(eventName, handler) {
        const handlers = this._getHandlersOfEvent(eventName);
        if (handlers) {
            handlers.add(handler);
        }
    }
    off(eventName, handler) {
        const handlers = this._getHandlersOfEvent(eventName);
        if (handlers) {
            handlers.delete(handler);
        }
    }
    trigger(eventName, ...args) {
        const handlers = this._getHandlersOfEvent(eventName);
        if (handlers) {
            let stopImmediate = false;
            handlers.forEach(handler => {
                if (stopImmediate) {
                    return;
                }
                try {
                    handler(...args, () => {
                        stopImmediate = true;
                    });
                }
                catch (error) {
                    console.error(error);
                }
            });
        }
    }
    _getHandlersOfEvent(eventName) {
        if (!this._handlerCollection.has(eventName)) {
            this._handlerCollection.set(eventName, new Set());
        }
        return this._handlerCollection.get(eventName);
    }
}
exports.TypedTrigger = TypedTrigger;
